@@ -8,33 +8,38 @@ corresponding proteins present in an annotation file, and can also
 use this annotation to include iBAQ measures.
 """
 
-import argparse, sys, re, csv, copy, decimal, itertools
-from multiprocessing import Pool
+import argparse, sys, re, csv, copy, decimal, itertools, os
+from multiprocessing import Pool, Value
 from collections import Counter
 from pythomics.templates import CustomParser
 import pythomics.proteomics.config as config
 import pythomics.proteomics.digest as digest
 import pythomics.parsers.fasta as fasta
+from pythomics.utils import ColumnFunctions
 
 parser = CustomParser(description = description)
 parser.add_fasta(help="The fasta file to match peptides against.")
 parser.add_out(help="The name of the file you wish to create with results appended.")
-parser.add_argument('--peptide_out', nargs='?', help="The file to write digested products to.", type=argparse.FileType('w'), default=sys.stdout)
-parser.add_argument('--protein_out', nargs='?', help="The file to write grouped products to.", type=argparse.FileType('w'), default=sys.stdout)
+parser.add_argument('--peptide-out', nargs='?', help="The file to write digested products to.", type=argparse.FileType('w'), default=os.devnull)
+parser.add_argument('--protein-out', nargs='?', help="The file to write grouped products to.", type=argparse.FileType('w'), default=os.devnull)
+parser.add_argument('--mod-out', nargs='?', help="The file to write a modification-centric summary to.", type=argparse.FileType('w'), default=os.devnull)
+parser.add_argument('--mod-col', help="The column to append for modifications (if you want to report ratios, etc.).", type=str)
+parser.add_argument('--mod-col-func', help="The function to apply to grouped entries in modification columns.", type=str, default='concat')
+parser.add_argument('--strict', help='For numeric operations, fail if types are incorrect (converting NA to a float for instance).', action='store_true')
 parser.add_delimited_file()
 parser.add_argument('-r', '--regex', help="A perl regular expression determining which parts of the header to capture.", type=str)
 parser.add_argument('--inferred-name', help="The name you want to assign for protein inference (in case you are regexing for gene names or something).", type=str, default='Proteins')
-parser.add_argument('--no-inference', help="Do not append proteins inferred from sequences.", action='store_false', default=False)
+parser.add_argument('--no-inference', help="Do not append proteins inferred from sequences.", action='store_false')
 group = parser.add_argument_group('iBAQ related options')
-group.add_argument('--ibaq', help="Provide to append iBAQ values as well (requires protein inference).", action='store_true', default=False)
+group.add_argument('--ibaq', help="Provide to append iBAQ values as well (requires protein inference).", action='store_true')
 group.add_argument('--precursors', help="The column with precursor area (defaults to header lines containing 'Precursor').", default=None)
 parser.add_enzyme()
-group.add_argument('--no-normalize', help="Don't normalize iBAQ to total intensity", action='store_false', default=True)
+group.add_argument('--no-normalize', help="Don't normalize iBAQ to total intensity", action='store_false')
 protein_group = parser.add_argument_group('Protein Grouping Options')
-protein_group.add_argument('--unique-only', help="Only group proteins with unique peptides", action='store_true', default=False)
-protein_group.add_argument('--position', help="Write the position of the peptide matches.", action='store_true', default=False)
-protein_group.add_argument('--case-sensitive', help="Treat peptides as case-sensitive (ie separate modified peptides)", action='store_true', default=False)
-protein_group.add_argument('--modification-site', help="Write the position in the parent protein of the modification (requires case-sensitive and modifications being lower-cased).", action='store_true', default=False)
+protein_group.add_argument('--unique-only', help="Only group proteins with unique peptides", action='store_true')
+protein_group.add_argument('--position', help="Write the position of the peptide matches.", action='store_true')
+protein_group.add_argument('--case-sensitive', help="Treat peptides as case-sensitive (ie separate modified peptides)", action='store_true')
+protein_group.add_argument('--modification-site', help="Write the position in the parent protein of the modification (requires case-sensitive and modifications being lower-cased).", action='store_true')
 
 
 global protein_sequences
@@ -52,7 +57,7 @@ def mapper(peptides):
     matched = {}
     for peptide, peptide_grouped in groups:
         indices = [(protein_sequences.count('\n', 0, match), match-protein_sequences[:match].rfind('\n')) for peptide, match in peptide_grouped]
-        found_proteins = [fasta_headers[i[0]] for i in indices]
+        found_proteins = list(set([fasta_headers[i[0]] for i in indices]))
         matched[peptide] = {'proteins': found_proteins, 'positions': [i[1] for i in indices], 'indices': [i[0] for i in indices]}
     return matched
 
@@ -61,15 +66,19 @@ def mapper(peptides):
 def main():
     global protein_sequences
     global fasta_headers
+    peptides_mapped = Value('i', 0)
     args = parser.parse_args()
     cores = args.p
     fasta_file = fasta.FastaIterator(args.fasta)
-    peptide_column = args.col-1
+    try:
+        peptide_column = int(args.col)-1
+    except:
+        peptide_column = None
     tsv_file = args.tsv
     out_file = args.out
     header_lines = args.header
     delimiter = args.delimiter
-    inference = not args.no_inference
+    inference = args.no_inference
     inferred_name = args.inferred_name
     digest_min = args.min
     digest_max = args.max
@@ -86,7 +95,7 @@ def main():
     precursor_columns = [int(i) for i in args.precursors.split(',')] if args.precursors else None
     if ibaq:
         enzyme = digest.Enzyme( enzyme=args.enzyme )
-    fasta_headers, protein_sequences = zip(*[(header, sequence) for header, sequence in fasta_file])
+    fasta_headers, protein_sequences = zip(*[(header.replace(';', ''), sequence) for header, sequence in fasta_file])
     #replace headers with parsed ones
     if args.regex:
         regex = re.compile(args.regex)
@@ -99,12 +108,27 @@ def main():
         cleaved = {}
     protein_sequences = '\n'.join(protein_sequences)
     peptide_history = {}
+    mod_grouping = {} # ordered by protein, site, type
     pep_count = 0
     pep_set = set([])
+    mod_col = args.mod_col
+    correlator = ColumnFunctions(args)
+    mod_col_func = getattr(correlator, args.mod_col_func, correlator.concat)
     with tsv_file as f:
         reader = csv.reader(f, delimiter=delimiter)
         for line_num, entry in enumerate(reader):
             if line_num < header_lines:#we assume the first header line is the one we care about
+                if peptide_column is None:
+                    for i,v in enumerate(entry):
+                        if v.lower() == args.col.lower():
+                            peptide_column = i
+                            break
+                if mod_col is not None and mod_col.isdigit():
+                    mod_col = int(mod_col)-1
+                elif mod_col is not None:
+                    for i,v in enumerate(entry):
+                        if v.lower() == args.mod_col.lower():
+                            mod_col = i
                 if not precursor_columns:
                         precursor_columns = [i for i, v in enumerate(entry) if 'precursor' in v.lower()]
                 if ibaq:
@@ -120,24 +144,30 @@ def main():
                     if entry[e_i]:
                         intensity = decimal.Decimal(entry[e_i])
                         try:
-                            peptide_history[peptide][n_i].add(intensity)
+                            peptide_history[peptide]['intensities'][n_i].add(intensity)
                         except KeyError:
-                            peptide_history[peptide] = {}
+                            peptide_history[peptide] = {'intensities': {}}
                             for i in xrange(len(precursor_columns)):
-                                peptide_history[peptide][i] = set([])
-                            peptide_history[peptide][n_i].add(intensity)
+                                peptide_history[peptide]['intensities'][i] = set([])
+                            peptide_history[peptide]['intensities'][n_i].add(intensity)
+                if mod_col is not None:
+                    try:
+                        peptide_history[peptide]['mod_col'] = entry[mod_col]
+                    except KeyError:
+                        peptide_history[peptide] = {'mod_col': entry[mod_col], 'intensities': {}}
         if ibaq and normalize:
             for peptide in peptide_history:
-                for i, v in peptide_history[peptide].iteritems():
+                for i, v in peptide_history[peptide]['intensities'].iteritems():
                     normalizations[i] += sum(v)
         else:
-            normalizations = [1 for peptide in peptide_history for intensities in peptide_history[peptide]]
+            normalizations = [1 for peptide in peptide_history for intensities in peptide_history[peptide]['intensities']]
 
     # map our peptides is a multi-cored manner
     pool = Pool(cores)
     # get our matches
-    plen = len(peptide_history)/cores+1
+    plen = 100#len(peptide_history)/cores+1
     peptides = peptide_history.keys()
+    # break into groups of 100 (empirically gives fastest mapping)
     subpeptides = [peptides[n:n+plen] for n in xrange(0, len(peptides), plen)]
     results = pool.map(mapper, subpeptides)
     mapped_peptides = dict((k, v) for d in results for (k, v) in d.items())
@@ -166,13 +196,13 @@ def main():
             try:
                 peptide_dict = peptide_grouping[peptide]
             except KeyError:
-                peptide_dict = {}
+                peptide_dict = {'intensities': {}}
                 peptide_grouping[peptide] = peptide_dict
             if not index%1000:
                 sys.stderr.write('%d of %d complete.\n' % (index, len(peptide_history)))
             mapped_info = mapped_peptides.get(peptide.upper(), empty_dict)
-            precursor_int = float(sum([sum(d[i]) for i in d]))
-            entry = [peptide, sum([len(d[i]) for i in d]), precursor_int]
+            precursor_int = float(sum([sum(d['intensities'][i]) for i in d['intensities']]))
+            entry = [peptide, sum([len(d['intensities'][i]) for i in d['intensities']]), precursor_int]
             if 'inference' not in peptide_dict:
                 peptide_dict['inference'] = {'proteins': False}
                 # if inference or ibaq:
@@ -203,14 +233,20 @@ def main():
                             mod_site_addition = []
                             for j, k in enumerate(peptide):
                                 if k.islower():
-                                    mod_site_addition.append('%s:%d'%(k,start_position+j))
+                                    mod_pos = start_position+j
+                                    mod_site_addition.append('%s:%d'%(k, mod_pos))
+                                    try:
+                                        mod_grouping[protein]['%s%d'%(k, mod_pos)]['values'].add(d['mod_col'])
+                                        mod_grouping[protein]['%s%d'%(k, mod_pos)]['peptides'].add(peptide)
+                                    except KeyError:
+                                        mod_grouping[protein] = {'%s%d'%(k, mod_pos): {'values': set([d['mod_col']]), 'peptides': set([peptide])}}
                             mod_site_additions.append('%s(%s)'%(protein,','.join(mod_site_addition)))
                         peptide_dict['inference']['mod_sites'] = ';'.join(mod_site_additions)
                     if out_position:
                         peptide_dict['inference']['matched_positions'] = ','.join(str(i) for i in start_positions)
             if ibaq:
                 ibaqs = []
-                intensities = [sum(d[i]) for i in d]
+                intensities = [sum(d['intensities'][i]) for i in d['intensities']]
                 if normalize:
                     precursor_int = sum([intensities[i]/normalizations[i] for i in xrange(len(normalizations))])
                     entry.append(precursor_int)
@@ -259,16 +295,19 @@ def main():
                                 #WBGene00004829(y:467,k:471);WBGene00019361(m:68);WBGene00019361(m:118);WBGene00019361(m:68);WBGene00020808(m:261);WBGene00020808(m:156)
                                 mod_prots = mod_protein.split(';')
                                 for mod_prot_ in mod_prots:
-                                    mod_prot, mod_prot_sites = mod_prot_.rsplit('(', 1)
-                                    if mod_prot == protein:
-                                        # print mod_prot_sites
-                                        for mod_prot_site in mod_prot_sites[:-1].split(','):
-                                            if mod_prot_site:
-                                                mod_aa, mod_prot_site = mod_prot_site[:-1].split(':')
-                                                mods.add((mod_aa, mod_prot_site))
+                                    try:
+                                        mod_prot, mod_prot_sites = mod_prot_.rsplit('(', 1)
+                                        if mod_prot == protein:
+                                            # print mod_prot_sites
+                                            for mod_prot_site in mod_prot_sites[:-1].split(','):
+                                                if mod_prot_site:
+                                                    mod_aa, mod_prot_site = mod_prot_site[:-1].split(':')
+                                                    mods.add((mod_aa, mod_prot_site))
+                                    except:
+                                        print 'failed', mod_prot_, mod_prots, mod_proteins
                     d = protein_grouping[protein][peptide]
-                    peptide_psm_count.append((peptide,sum([len(d[i]) for i in d])))
-                    intensities += [sum(d[i]) for i in d]
+                    peptide_psm_count.append((peptide,sum([len(d['intensities'][i]) for i in d['intensities']])))
+                    intensities += [sum(d['intensities'][i]) for i in d['intensities']]
                     if ibaq and normalize:
                         precursor_int += sum([intensities[i]/normalizations[i] for i in xrange(len(normalizations))])
                 entry.append(';'.join(['%s(%s)' % (i,j) for i,j in peptide_psm_count]))
@@ -341,6 +380,20 @@ def main():
                             entry.append(';'.join(mod_entry))
                 out_writer.writerow(entry)
         stats['modifications'] = mod_stats
+    if args.mod_out:
+        with args.mod_out as o:
+            writer = csv.writer(o, delimiter=delimiter)
+            header = ['Site', inferred_name, 'Peptide']
+            if mod_col:
+                header.append(args.mod_col)
+            writer.writerow(header)
+            #mod_grouping[protein] = {'%s%d'%(k, mod_pos): {'values': set([d['mod_col']]), 'peptides': set([peptide])}}
+            for protein, sites_dict in mod_grouping.iteritems():
+                for site, site_dict in sites_dict.iteritems():
+                    entry = [site, protein, ';'.join(site_dict.get('peptides'))]
+                    if mod_col:
+                        entry.append(mod_col_func(site_dict.get('values',[])))
+                    writer.writerow(entry)
     # write stats
     sys.stderr.write('Peptides Searched: %s\n'%stats['peptides'])
     sys.stderr.write('Unique Peptides Found: %s\n'%stats['peptides_found'])
